#!/bin/bash

PRODUCT=mvBlueFOX
TARNAME=mvbluefox
USER=$(whoami)

if [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
   echo
   echo "Installer script for the $PRODUCT driver."
   echo
   echo " Usage : $0 [directory_name] [version]"
   echo
   echo "Optional argument 'directory_name' specifies where to place the driver."
   echo 'If the directory does not yet exist, it will be created.'
   echo
   echo " Example : $0 $HOME/matrix-vision/$PRODUCT"
   echo
   echo "Optional argument 'version' specifies which .tgz file to unpack."
   echo 'This is only relevant when the archive directory contains multiple'
   echo "versions of $PRODUCT-n.n.n.tgz."
   echo "If this is so and 'version' is not specified on the command line, the"
   echo 'latest version will be unpacked.'
   echo
   echo 'To install in the current working directory, simply leave out the'
   echo "directory name, or specify '.' So the two commands below do the same:"
   echo
   echo " $0 1.10.33"
   echo " $0 . 1.10.33"
   echo
   echo "To summarize: 'directory_name' defaults to the current working directory"
   echo "and 'version' defaults to the latest version. Consequently, if no"
   echo 'arguments are passed to the script, the latest version is installed'
   echo 'in the current working directory.'
   echo
   echo "This script must be in the directory that contains $PRODUCT-n.n.n.tgz"
   echo
   exit
fi

if [ "$1" == "." ] || [ "$1" == "./" ]; then
   set $PWD $2
elif ! $(echo $1 | grep -q [a-zA-Z]); then
   # Nothing or just the version (e.g. 1.10.30) was specified
   # on the command line, i.e. no installation path was specified.
   # Install in the current working directory.
   # Without this test, if just the version was specified, that
   # (e.g. 1.10.30) would become the name of the newly created
   # directory, and the version would no longer be explicitly specified.
   # We assume that the directory name will contain at least one character.
   set $PWD $1
   echo
   echo 'No target directory specified'
   echo 'Will use the current working directory'
else
   PLEN=`expr length $1`
   if [ "`expr substr $1 $PLEN $PLEN`" == "/" ]; then
      # strip the trailing / and reassign to positional argument one
      # (which can only be done using 'set')
      PLAST=`expr $PLEN - 1`
      set `expr substr $1 1 $PLAST` $2
   fi

   if [ "`expr substr $1 1 1`" != "/" ]; then
      # make an absolute path out of a relative one and reassign
      # to positional argument one (which can only be done using 'set')
      set $PWD/$1 $2
   fi
fi

TARGET=$(uname -m)
if [ "$TARGET" == "i686" ]; then
   TARGET=x86
fi

# TESTVERARG=$1
# while $(echo $TESTVERARG | grep -q '\.'); do TESTVERARG=`echo $TESTVERARG | sed -e s/\\\.//`;done

LIBVER=$2

echo
if which dirname >/dev/null; then
    SCRIPTSOURCEDIR=$(dirname $0)
fi
if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
   if [ -z $SCRIPTSOURCEDIR ] || [ "$SCRIPTSOURCEDIR" == "." ]; then
      SCRIPTSOURCEDIR=$PWD
   else
      cd $SCRIPTSOURCEDIR
      if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
         echo "Could not cd to sourcedir"
         echo
      fi
   fi
fi

# check for ld.so.conf
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
   LDPATH=/etc/ld.so.conf
else
   LDPATH=/etc/ld.so.conf
fi

# move to the destination directory
if ! [ -d $1 ]; then
   # the destination directory does not yet exist
   # first try to create it as a normal user
   mkdir --parent $1 >/dev/null 2>&1
   if ! [ -d $1 ]; then
      # that didn't work
      # now try it as superuser
      sudo mkdir --parent $1
   fi
   if ! [ -d $1 ]; then
      echo 'Error : could not create target directory' $1.
      echo 'Maybe you specified a partition that was mounted read only.'
      echo
      exit
   fi
fi

# now unpack the actual libraries
if ls mvbluefox* > 0; then
   TARNAME=$TARNAME
elif ls beta_mvbluefox* > 0; then
   echo "No release build found, will now use beta_*.tgz"
   TARNAME="beta_"$TARNAME
else
  echo "No matching mvbluefox*.tgz, beta_mvbluefox*.tgz."
  echo "Please go sure to put the tgz in the same directory as this script."
fi

if ! [ -z $LIBVER ] && [ -r $TARNAME-$TARGET-$LIBVER.tgz ]; then
   TARFILE=$TARNAME-$TARGET-$LIBVER.tgz
else
   for a in $TARNAME-$TARGET-*.tgz; do TARFILE=$a; done
fi

if ! [ -r $TARFILE ]; then
   echo 'Error : could not read' $TARFILE.
   echo
   exit
fi

# for tarfile mv*-x86-1.10.30.tgz, VERSION would be 1.10.30
if [ $TARNAME == "mvbluefox" ] || [ $TARNAME == "beta_mvbluefox" ]; then
   VERSION=`echo $TARFILE | sed -e s/$TARNAME-$TARGET_ABI2-// | sed -e s/\\.tgz//`
else
   VERSION=`echo $TARFILE | sed -e s/$TARNAME-$TARGET-// | sed -e s/\\.tgz//`
fi

# needed at compile time (used during development, but not shipped with the final program)
AC=mvIMPACT_acquire
if [ $TARNAME == "mvbluefox" ] || [ $TARNAME == "beta_mvbluefox" ]; then
   ACT=$AC-$TARGET-$VERSION
else
   ACT=$AC-$VERSION
fi

if ! [ -z $LIBVER ] && [ "$LIBVER" != "$VERSION" ]; then
   echo 'Warning : requested version' $LIBVER, 'found version' $VERSION.
   echo
fi

cd /tmp
sudo tar xfz $SCRIPTSOURCEDIR/$TARFILE

cd $1
# remove older versions (if any)
rm -f lib/libmv*.so*

if [ -d $1 ]; then
   cp -f -R /tmp/$ACT /$1
else
   cp -f -R /tmp/$ACT $HOME
fi

cd $1/$ACT

# set ld.so.conf 
if grep -q $1/$ACT/lib/$TARGET /etc/ld.so.conf; then
   echo "$1/$ACTlib/$TARGET already defined in /etc/ld.so.conf."
else
   sudo sh -c "echo '$1/$ACT/lib/$TARGET' >> /etc/ld.so.conf"
fi

sudo /sbin/ldconfig

# install needed libraries and compliler
COULD_NOT_INSTALL="Found neither apt-get nor yast; please install >%s< manually."

# check if we have g++
if ! which g++ >/dev/null 2>&1; then
   if which apt-get >/dev/null 2>&1; then
      sudo apt-get -q install g++
   elif sudo which yast >/dev/null 2>&1; then
      YASTBIN=`sudo which yast`
      sudo $YASTBIN --install gcc-c++
   else
      printf "$COULD_NOT_INSTALL" "g++"
   fi
fi

if ! which make >/dev/null 2>&1; then
   if sudo which yast >/dev/null 2>&1; then
      YASTBIN=`sudo which yast`
      sudo $YASTBIN --install make
   else
      printf "$COULD_NOT_INSTALL" "make"
   fi
fi

# check if we have libexpat
if ! /sbin/ldconfig -p | grep -q "expat"; then
   if which apt-get >/dev/null 2>&1; then
      sudo apt-get -q install libexpat1-dev
   elif sudo which yast >/dev/null 2>&1; then
      YASTBIN=`sudo which yast`
      sudo $YASTBIN --install expat libexpat-devel
   else
      printf "$COULD_NOT_INSTALL" "libexpat"
   fi
fi

INPUT_REQUEST="Do you want to install >%s< (default is 'yes')?\nHit 'n' + <Enter> for 'no', or just <Enter> for 'yes'.\n"
YES_NO=

# do we want to install wxWidgets?
if ! which wx-config >/dev/null 2>&1; then
   echo
   printf "$INPUT_REQUEST" "wxWidgets"
   echo "This is highly recommended, as without wxWidgets, you cannot build wxPropView."
   echo
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo 'Not installing wxWidgets'
   else
      if which apt-get >/dev/null 2>&1; then
         echo 'Installing wxWidgets'
         sudo apt-get -q install libwxgtk2.8-dev libwxbase2.8-0 libwxbase2.8-dev libwxgtk2.8-0 wx2.8-headers build-essential libgtk2.0-dev
      elif sudo which yast >/dev/null 2>&1; then
         echo 'Installing wxWidgets'
         YASTBIN=`sudo which yast`
         sudo $YASTBIN --install wxGTK-devel
      else
         printf "$COULD_NOT_INSTALL" "wxWidgets"
      fi
   fi
fi

# do we want to install FLTK?
if ! which fltk-config >/dev/null 2>&1; then
   echo
   printf "$INPUT_REQUEST" "FLTK"
   echo "This is only required if you want to build the 'LiveSnapFLTK' sample."
   echo
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo 'Not installing FLTK'
   else
      if which apt-get >/dev/null 2>&1; then
         echo 'Installing FLTK'
         sudo apt-get -q install libgl1-mesa-dev
         sudo apt-get -q install libfltk1.1-dev
      elif sudo which yast >/dev/null 2>&1; then
         echo 'Installing FLTK'
         YASTBIN=`sudo which yast`
         sudo $YASTBIN --install Mesa-devel
         sudo $YASTBIN --install fltk-devel
      else
         printf "$COULD_NOT_INSTALL" "FLTK"
      fi
   fi
fi

echo
echo "Do you want the tools and samples to be built (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
   echo 'The tools and samples were not built.'
   echo 'To build them yourself, type:'
   echo '  cd ~/mvimpact-acquire'
   echo "  make $TARGET"
   echo '  sudo /sbin/ldconfig'
else
   make $TARGET
   sudo /sbin/ldconfig
fi

echo
echo "Do you want to copy 51-mvbf.rules to /etc/udev/rules.d for non-root user support (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
   echo 
   echo 'To grand non-root user support,'
   echo 'copy 51-mvbf.rules the file to /etc/udev/rules.d'
   echo 
else
   sudo cp -f $1/$ACT/Scripts/51-mvbf.rules /etc/udev/rules.d
fi

# check if plugdev group exists and users is member of it
echo
if ! grep -q plugdev /etc/group; then
   echo "Group 'plugdev' don't exists, this is necessary to run as non-root user, do you want to create it"
   echo "and add users to 'plugdev' (default is 'yes')?"
   echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo
      echo "'plugdev' will be not created and you can't run the device as non-root user!"
      echo "If you want non-root users support, you will need to create 'plugdev'"
      echo "and add the users to this group."
   else
      sudo /usr/sbin/groupadd -g 46 plugdev
      sudo /usr/sbin/usermod $USER -A plugdev
      echo "Group 'plugdev' created and user '"$USER"' added to."
   fi
fi

echo
echo "---------------------------------------------------------------"
echo "Installation successful!"
echo "---------------------------------------------------------------"
echo
if ! grep -q plugdev /etc/group; then
   echo "Do you want to reboot now /deafult is 'yes')?"
   echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo "You need to reboot manually to complete the installation."
   else
      sudo /usr/sbin/reboot
   fi
fi




